﻿-- Author      : Celess
-- Create Date : 11/8/2008 8:59:19 AM

RaidTracker_Export = {

}

function RaidTracker_Export:GenerateRaid( raid )
	local db = KARaidTrackerDB;
	
	if type(raid) ~= "table" then
		KARaidTracker:Print("Bug: Invalid raid table passed to export, please report.");
		return;
	end	
	if not raid.End then
		KARaidTracker:Print("You have to end the raid before you exporting it");
		return;
	end;

	if db.Options.ExportFormat == 1 then
		return self:GenerateMLdkpXML(raid);
	end;
	
	return self:GenerateDkpXML(raid);
end

-- helper functions
function RaidTracker_Export:GetDate( s )
	return (KARaidTrackerDB.Options.ExportFormat == 0) and s or KARaidTracker:GetTimestamp(s)
end

function RaidTracker_Export:GenerateMLdkpXML( raid )
    local db = KARaidTrackerDB
    local options = db.Options
    local format = options.ExportFormat

	local race, class, level, sex;
	
	
	local xml ='<?xml version="1.0"?>';
	xml = xml..'<!DOCTYPE ML_Raidtracker PUBLIC "-//MLdkp//DTD ML_Raidtracker V 1.5//EN" "http://www.mldkp.net/dtds/1.0/ML_Raidtracker.dtd">';
	
	xml = xml.."<raidinfo>";
	xml = xml.."<version>1.5</version>";
	
	xml = xml.."<start>"..self:GetDate(raid.key).."</start>";
	xml = xml.."<end>"..self:GetDate(raid.End).."</end>";

	if raid.zone then
		xml = xml.."<zone>"..raid.zone.."</zone>";
	end
	if raid.instanceid then
		xml = xml.."<instanceid>"..raid.instanceid.."</instanceid>";
	end
	xml = xml.."<exporter>"..UnitName("Player").."</exporter>";
	
	if raid.PlayerInfos then
		xml = xml.."<playerinfos>";
		for key, val in pairs(raid.PlayerInfos) do
			xml = xml.."<player>";
			xml = xml.."<name>"..key.."</name>";
			for key2, val2 in pairs(raid.PlayerInfos[key]) do
				if key2 == "note" then
					xml = xml.."<"..key2.."><![CDATA["..val2.."]]></"..key2..">";
				elseif key2 == "class" then
					xml = xml.."<"..key2..">"..KARaidTracker.ClassToIDTable[val2].."</"..key2..">";
				elseif key2 == "race" then
					xml = xml.."<"..key2..">"..KARaidTracker.RaceToIDTable[val2].."</"..key2..">";
				elseif key2 == "level" then
					if db.Options.MaxLevel ~= val2 then
						xml = xml.."<"..key2..">"..val2.."</"..key2..">";
					end;
				else
					xml = xml.."<"..key2..">"..val2.."</"..key2..">";
				end
			end
			xml = xml.."</player>";
		end
		xml = xml.."</playerinfos>";
	end
	if raid.BossKills then
		local bosskillsindex = 1;
		xml = xml.."<bosskills>";
		for key, val in pairs(raid.BossKills) do
			xml = xml.."<bosskill>";
			xml = xml.."<name>"..val.boss.."</name>";
			xml = xml.."<time>"..self:GetDate(val.time).."</time>";
			xml = xml.."</bosskill>";
		end
		xml = xml.."</bosskills>";
	end
	if raid.wipes then
		xml = xml.."<wipes>";
		for key, val in pairs(raid.wipes) do
			xml = xml.."<wipe><time>"..val.."</time></wipe>";
		end
		xml = xml.."</wipes>";
	end
	if raid.bossnext then
		xml = xml.."<nextboss>"..raid.bossnext.."</nextboss>";
	end

	if raid.note then 
		xml = xml.."<note><![CDATA["..raid.note.."]]></note>"; 
	end

	xml = xml.."<joins>";
	for key, val in pairs(raid.Join) do
		xml = xml.."<join>";
		xml = xml.."<player>"..val.player.."</player>";
		xml = xml.."<time>"..self:GetDate(val.time).."</time>";
		xml = xml.."</join>";
	end
	xml = xml.."</joins>";
	xml = xml.."<leaves>";
	for key, val in pairs(raid.Leave) do
		xml = xml.."<leave>";
		xml = xml.."<player>"..val.player.."</player>";
		xml = xml.."<time>"..self:GetDate(val.time).."</time>";
		xml = xml.."</leave>";
	end
	xml = xml.."</leaves>";
	xml = xml.."<loots>";
	
	for key, val in pairs(raid.Loot) do
		xml = xml.."<loot>";
		xml = xml.."<itemname>"..val.item.name.."</itemname>";
		xml = xml.."<itemid>"..val.item.id.."</itemid>";
		xml = xml.."<count>"..val.item.count.."</count>";
		xml = xml.."<player>"..val.player.."</player>";
		if val.costs then
			xml = xml.."<costs>"..val.costs.."</costs>";
		end
		xml = xml.."<time>"..self:GetDate(val.time).."</time>";
		if val.zone then 
			xml = xml.."<zone>"..val.zone.."</zone>";
		end
		if val.boss then 
			xml = xml.."<boss>"..val.boss.."</boss>";
		end
		if val.note then 
			xml = xml.."<note><![CDATA["..val.note.."]]></note>";
		end
		xml = xml.."</loot>";
	end
	xml = xml.."</loots>";
	xml = xml.."</raidinfo>";
	
	return xml;
end


function RaidTracker_Export:GenerateDkpXML( raid )
    local db = KARaidTrackerDB
    local options = db.Options
    local format = options.ExportFormat
	
	local xml = "<RaidInfo>"
	if format == 0 then
		local xml = xml.."<Version>1.4</Version>";
	end
	
	if format == 0 then
		xml = xml.."<key>"..raid.key.."</key>";
	end
	xml = xml.."<start>"..self:GetDate(raid.key).."</start>";
	
	if raid.End then
		xml = xml.."<end>"..self:GetDate(raid.End).."</end>";
	end
	if raid.zone then
		xml = xml.."<zone>"..raid.zone.."</zone>";
	end
	if raid.PlayerInfos then
		xml = xml.."<PlayerInfos>";
		local idx = 1;
		for key, val in pairs(raid.PlayerInfos) do
			xml = xml.."<key"..idx..">";
			xml = xml.."<name>"..key.."</name>";
			for key2, val2 in pairs(raid.PlayerInfos[key]) do
				if key2 == "note" then
					xml = xml.."<"..key2.."><![CDATA["..val2.."] ]></"..key2..">";
				
				else
					xml = xml.."<"..key2..">"..val2.."</"..key2..">";
				end
			end
			xml = xml.."</key"..idx..">";
			idx = idx + 1;
		end
		xml = xml.."</PlayerInfos>";
	end
	if raid.BossKills then
		local bosskillsindex = 1;
		xml = xml.."<BossKills>";
		for key, val in pairs(raid.BossKills) do
			xml = xml.."<key"..bosskillsindex..">";
			xml = xml.."<name>"..val.boss.."</name>";
			xml = xml.."<time>"..self:GetDate(val.time).."</time>";
			if format ~= 1 then
				xml = xml.."<time>"..val.time.."</time>";
				if  raid.BossKills[key].attendees then
					xml = xml.."<attendees>";
					for key2, val2 in pairs(raid.BossKills[key].attendees) do
						xml = xml.."<key"..key2..">";
						xml = xml.."<name>"..val2.."</name>";
						xml = xml.."</key"..key2..">";
					end
					xml = xml.."</attendees>";
				end
			end
			xml = xml.."</key"..bosskillsindex..">";
			bosskillsindex = bosskillsindex + 1;
		end
		xml = xml.."</BossKills>";
	end
	-- new exports
	if raid.wipes then
		xml = xml.."<Wipes>";
		for key, val in pairs(raid.wipes) do
			xml = xml.."<Wipe>"..val.."</Wipe>";
		end
		xml = xml.."</Wipes>";
	end
	if raid.bossnext then
		xml = xml.."<NextBoss>"..raid.bossnext.."</NextBoss>";
	end
	--	
	if format == 0 then
		local sNote = "<note><![CDATA[";
		if raid.note then sNote = sNote..raid.note; end
		if raid.zone then sNote = sNote.." - Zone: "..raid.zone; end
		sNote = sNote.."]]></note>";
		xml = xml..sNote;
	else
		if raid.note then xml = xml.."<note><![CDATA["..raid.note.."]]></note>"; end
	end
	xml = xml.."<Join>";
	for key, val in pairs(raid.Join) do
		xml = xml.."<key"..key..">";
		xml = xml.."<player>"..val.player.."</player>";
		if format == 0 then
			if val.race then
				race = val.race; 
			elseif raid.PlayerInfos and raid.PlayerInfos[val.player] and raid.PlayerInfos[val.player].race then
				race = raid.PlayerInfos[val.player].race; 
			else
				race = nil;
			end
			if val.class then
				class = val.class; 
			elseif raid.PlayerInfos and raid.PlayerInfos[val.player] and raid.PlayerInfos[val.player].class then
				class = raid.PlayerInfos[val.player].class; 
			else
				class = nil;
			end
			if val.sex then
				sex = val.sex; 
			elseif raid.PlayerInfos and raid.PlayerInfos[val.player] and raid.PlayerInfos[val.player].sex then
				sex = raid.PlayerInfos[val.player].sex; 
			else
				sex = nil;
			end
			if val.level then
				level = val.level; 
			elseif raid.PlayerInfos and raid.PlayerInfos[val.player] and raid.PlayerInfos[val.player].level then
				level = raid.PlayerInfos[val.player].level; 
			else
				level = nil;
			end
			if race then xml = xml.."<race>"..race.."</race>"; end
			if class then xml = xml.."<class>"..class.."</class>"; end
			if sex then xml = xml.."<sex>"..sex.."</sex>"; end
			if level then xml = xml.."<level>"..level.."</level>"; end
		end
		if format == 0 and raid.PlayerInfos[val.player] and raid.PlayerInfos[val.player].note then xml = xml.."<note>"..raid.PlayerInfos[val.player].note.."</note>"; end
		xml = xml.."<time>"..self:GetDate(val.time).."</time>";
		xml = xml.."</key"..key..">";
	end
	xml = xml.."</Join>";
	xml = xml.."<Leave>";
	for key, val in pairs(raid.Leave) do
		xml = xml.."<key"..key..">";
		xml = xml.."<player>"..val.player.."</player>";
		xml = xml.."<time>"..self:GetDate(val.time).."</time>";
		xml = xml.."</key"..key..">";
	end
	xml = xml.."</Leave>";
	xml = xml.."<Loot>";
	for key, val in pairs(raid.Loot) do
		xml = xml.."<key"..key..">";
		xml = xml.."<ItemName>"..val.item.name.."</ItemName>";
		xml = xml.."<ItemID>"..val.item.id.."</ItemID>";
		if val.item.icon then xml = xml.."<Icon>"..val.item.icon.."</Icon>"; end
		if val.item.class then xml = xml.."<Class>"..val.item.class.."</Class>"; end
		if val.item.subclass then xml = xml.."<SubClass>"..val.item.subclass.."</SubClass>"; end
		xml = xml.."<Color>"..val.item.c.."</Color>";
		xml = xml.."<Count>"..val.item.count.."</Count>";
		xml = xml.."<Player>"..val.player.."</Player>";
		if val.costs then
			xml = xml.."<Costs>"..val.costs.."</Costs>";
		end
		xml = xml.."<Time>"..self:GetDate(val.time).."</Time>";
		if val.zone then xml = xml.."<Zone>"..val.zone.."</Zone>"; end
		if val.boss then xml = xml.."<Boss>"..val.boss.."</Boss>"; end
		if format == 0 then
			local sNote = "<Note><![CDATA[";
			if val.note then sNote = sNote..val.note; end
			if val.zone then sNote = sNote.." - Zone: "..val.zone; end
			if val.boss then sNote = sNote.." - Boss: "..val.boss; end
			if val.costs then sNote = sNote.." - "..val.costs.." DKP"; end
			sNote = sNote.."]]></Note>";
			xml = xml..sNote;
		else
			if val.note then xml = xml.."<Note><![CDATA["..val.note.."]]></Note>"; end
		end
		xml = xml.."</key"..key..">";
	end
	xml = xml.."</Loot>";
	xml = xml.."</RaidInfo>";
	
	return xml;
end

