﻿if KarmaAce then return; end

--[[
	Simple lightweight support for other libraries and systems
	
	Little complicated because needs to be tolerant of libs not existing and still provide the most
	of what it can for whatever loaded libs exist. The theroy is that an app could just include karma
	and no other libs embedded, ondemand, or optional and just works. If fubar exists then it just
	works so long as ace3 is around for example, or ct mod, or acelocale. There is no wrestling with
	ac3 vs rock vs fubar, thats done once here and insulates the app. Locale is unified so that even
	the karma libs use the same instance of locales as the app, as the karma base, or the aceaddon
	instance. Puts the app alot closer to bare metal itself because it just doesn't have to care, and
	still get benifits; take, leave, or be blissfully unaware.
	
	Currently provides some support for fubar, aceaddon, ct mod, acelocale, and a few of the babble.
	Intend to sew bables with base locale so if babble exists it just makes them one lookup, and same
	for the reverse lookups.
]]

KarmaAce = {
	_locale_mt = {
		__newindex	= function( t, k, v )	end,	-- do nothing, bad... no write
		__index		= function( t, k ) 
			local found; 
			for i, v in pairs(t) do 
				v = v and v[k] or nil;
				if v then 
					found = true;
					if v~=k then
						return v;
					end;
				end;
			end;
			if not found then
--				KarmaLibrary:SysMsg("not found(",k,")");
			end
			return k;
		end,
	},
	_locale_proxy	= { __index = function( t, k ) return rawget(rawget(t,1),k) end, },
	_locale_proxyr	= { __index = function( t, k ) for i, v in pairs(rawget(t,1)) do if v==k then return i end end end, },
}

function KarmaAce:OnVariablesLoadedAce( frame, inst )
	inst._acelib = {
		name	= inst._instance.name.."Ace",
		nameDB	= inst._instance.name.."AceDB",
		lib = {
			L	= LibStub and LibStub("AceLocale-3.0", true),
			LZ	= LibStub and LibStub("LibBabble-Zone-3.0", true),
			LB	= LibStub and LibStub("LibBabble-Boss-3.0", true),
			fu	= LibStub and LibStub("LibFuBarPlugin-3.0", true);
			aao	= LibStub and LibStub("AceAddon-3.0", true);
			db	= LibStub and LibStub("AceDB-3.0", true);
		},
		locale = { }, L = { }, LR = { },
	};
	local _inst = inst._acelib;
	_inst.locale.base	= _inst.lib.L	and _inst.lib.L:GetLocale(inst._instance.name, true);
	_inst.locale.zone	= _inst.lib.LZ	and _inst.lib.LZ:GetUnstrictLookupTable();
	_inst.locale.boss	= _inst.lib.LB	and _inst.lib.LB:GetUnstrictLookupTable();
	_inst.locale.zoneR	= _inst.lib.LZ	and _inst.lib.LZ:GetReverseLookupTable();
	_inst.locale.bossR	= _inst.lib.LB	and _inst.lib.LB:GetReverseLookupTable();
	
	-- sew the locale lookups together, forward and reverse
	--   sadly have to put base in a proxy for the moment because is wired to acelocale
	local t = _inst.locale;
	_inst.L = setmetatable({t.zone, t.boss, setmetatable({t.base},self._locale_proxy)}, self._locale_mt);
	_inst.LR = setmetatable({t.zoneR, t.bossR, setmetatable({t.base},self._locale_proxyr)}, self._locale_mt);
	
	--load the locales for this lib
	if KarmaLocale and KarmaLocale.OnVariablesLoaded and _inst.lib.L then
		KarmaLocale:OnVariablesLoaded( frame, inst, _inst.lib.L, inst._instance.name );
	end
end


function KarmaAce:OnPlayerLoginAce( frame, inst )
	local _inst = inst._acelib;

	_inst.instance	= _inst.lib.aao and _inst.lib.aao:NewAddon(_inst.name, "AceConsole-3.0");
		if _inst.instance and _inst.lib.fu then _inst.lib.aao:EmbedLibrary(_inst.instance, "LibFuBarPlugin-3.0") end
	_inst.db		= _inst.lib.db	and _inst.lib.db:New(_inst.nameDB, { profile = {
		showwindows = { res = false, status = false, },
		rsinput		= { show = false, xOfs = 0, yOfs = 0, saved = false, },
	} } );
	_inst.menu		= _inst.L and _inst.instance and _inst.lib.fu and { handler = _inst.instance, type = "group", args = {
		help	= {	type = "execute", order = 10, name = _inst.L["Help"], desc = _inst.L["Show Help"],
					passValue = _inst.instance, func = function(self) self:ExecChatCommandAce("help") end,		},
		main	= {	type = "execute", order = 10, name = _inst.L["Main Window"], desc = _inst.L["Show Main Window"],
					passValue = _inst.instance, func = function(self) self:ExecChatCommandAce() end,			},
		options	= {	type = "execute", order = 10, name = _inst.L["Options"], desc = _inst.L["Show Options"],
					passValue = _inst.instance, func = function(self) self:ExecChatCommandAce("options") end,	},
	} };

	-- ace and related
	if _inst.instance then										-- set up options
		local aceInst = _inst.instance;
		aceInst._karma					= inst;
		aceInst._L						= _inst.L;
		aceInst.db						= _inst.db;
		aceInst.title					= inst._addonmeta.title;
		aceInst.version					= inst._addonmeta.version;
		aceInst.OnUpdateFuBarTooltip	= KarmaAce.OnUpdateFuBarTooltip;	-- Mouseover menu
		aceInst.OnFuBarClick			= KarmaAce.OnFuBarClick
		aceInst.ExecChatCommandAce		= KarmaAce.ExecChatCommandAce;
		aceInst.OnMenuRequest			= _inst.menu;
		aceInst.OnInitialize			= KarmaAce.OnInitialize;
		aceInst:SetFuBarOption("hasIcon", true)
		aceInst:SetFuBarOption("iconPath", inst._addonmeta.icon);
		aceInst:SetFuBarOption("hasNoColor", true)
		aceInst:SetFuBarOption("independentProfile", true)
		aceInst:SetFuBarOption("clickableTooltip", true)
		aceInst:SetFuBarOption("cannotDetachTooltip", true)
		aceInst:SetFuBarOption("hideWithoutStandby", true)
		aceInst:SetFuBarOption("defaultPosition", "CENTER")
		aceInst:SetFuBarOption("configType", "Dewdrop-2.0")
--		_inst.lib.fu:OnEmbedInitialize(aceInst);
--		inst:DebugTable("OnPlayerLoginAce: _inst", aceInst);
	end

	-- ct mod and related
	if CT_RegisterMod then
		CT_RegisterMod(inst._addonmeta.title, "Display window", 5, self._addonmeta.icon,
			self._addonmeta.desc, "switch", self, function(self) self:ExecChatCommandAce() end );
	end

end


function KarmaAce:OnUpdateFuBarTooltip()
	GameTooltip:AddLine(self._karma._instance.name);
	GameTooltip:AddLine(" ");
	if self.OnUpdateFuBarTooltipCustom then self:OnUpdateFuBarTooltipCustom(); end
	GameTooltip:AddLine(self._L["SHOW_TOOLTIP_FU"], 0.2, 1, 0.2, 1);
end


function KarmaAce:OnFuBarClick(button)
	self:ExecChatCommandAce( IsShiftKeyDown() and "options" )
end


function KarmaAce:ExecChatCommandAce( command )
	local cmd = (command and " "..command) or "";
	self._karma:ExecuteChatCommand( "/"..self._karma._instance.name..cmd );
end


function KarmaAce:OnInitialize( )
	local _inst = self._karma._acelib;
	if _inst.menu then
		self._karma:ArgsToTable(_inst.menu.args, _inst.lib.fu:GetEmbedRockConfigOptions(_inst.instance));
		for i, v in pairs (_inst.menu.args) do
			if v.type == "boolean" then v.type = "toggle" end
			if v.type == "choice" then v.type = "text" end
			if v.choices then v.validate = v.choices end
			if i == "hide" then v.name = "Hide" end
		end
	end
end
