﻿
-- Globals
KA_RaidTracker_LastWipe = 0;

local KA_RaidTracker_Events = { };
local KA_RaidTracker_AutoBossChangedTime = 0;
local KA_RaidTracker_TimeOffset = 0;
local KA_RaidTracker_GameTimeFrame_Update_Original = nil;

function KARaidTracker:OnLoad( frame )
	RaidTrackerFrameHeaderTitle:SetText( self._addonmeta.title );
	RaidTrackerFrameHeaderVersion:SetText( "|c0099cfffv" .. self._addonmeta.version .. " "..self._addonmeta.locale.."|r" );
	UIPanelWindows["RaidTrackerFrame"] = { area = "left", pushable = 1, whileDead = 1 };

	-- Register events
	frame:RegisterEvent("CHAT_MSG_LOOT");
	frame:RegisterEvent("CHAT_MSG_SYSTEM");
	frame:RegisterEvent("RAID_ROSTER_UPDATE");
	frame:RegisterEvent("VARIABLES_LOADED");
	frame:RegisterEvent("UPDATE_MOUSEOVER_UNIT");
	frame:RegisterEvent("ZONE_CHANGED_NEW_AREA");
	frame:RegisterEvent("CHAT_MSG_COMBAT_HOSTILE_DEATH");
	frame:RegisterEvent("CHAT_MSG_MONSTER_YELL");
	frame:RegisterEvent("CHAT_MSG_MONSTER_EMOTE");
	frame:RegisterEvent("PLAYER_LOGIN");
	frame:RegisterEvent("PLAYER_ENTERING_WORLD");
	frame:RegisterEvent("PARTY_MEMBERS_CHANGED");
	frame:RegisterEvent("UNIT_HEALTH");
	frame:RegisterEvent("UPDATE_INSTANCE_INFO");

	SlashCmdList["RAIDTRACKER"] = KARaidTracker_CommandOptions;
	SLASH_RAIDTRACKER1 = "/raidtracker";
	SLASH_RAIDTRACKER2 = "/rt";
end


function KARaidTracker:OnVariablesLoaded( frame )
	RaidTrackerFrame:CreateTitleRegion()
	RaidTrackerFrame:GetTitleRegion():SetAllPoints()
	KA_RaidTracker_GetGameTimeOffset();
end


function KARaidTracker:OnPlayerLogin( frame )
	local _inst = self._acelib;
	if _inst.menu then
		local args = _inst.menu.args;
		args.itemoptions = {
			type = "execute", order = 11,
			name = _inst.L["Item Options"], desc = _inst.L["Show Item Options"],
			passValue = _inst.instance, func = function(self) self:ExecChatCommandAce("io") end,
		};
	end
end


function KARaidTracker_CommandOptions(msg)
	KARaidTracker:CommandOptions(msg)
end

function KARaidTracker:CommandOptions(msg)
	local db = self._store.db;
	local options = self._options;
	local frame = RaidTrackerFrame
	local _, _, command, args = string.find(msg, "(%w+)%s?(.*)");
	if command then
		command = strlower(command);
	end

	if command == "debug" then
		if args=="1" then
			options.DebugMode = 1
		elseif args=="0" then
			options.DebugMode = 0
		end
		self:Print("Debug Mode: ".. ((options.DebugMode and "Enabled") or "Disabled"));
	elseif command == "addwipe" then
		KA_RaidTracker_AddWipe();
	elseif command == "deleteall" then
		db.RaidLog = { };
		options.CurrentRaid = nil;
		self:Print("Deleted "..getn(db.RaidLog).." Raids");
		KA_RaidTracker_Update();
		KA_RaidTracker_UpdateView();
	elseif command == "additem" then
		local linkMatch = "|c%x+|Hitem:[-%d:]+|h%[.-%]|h|r"
		local _, _, item, player = string.find(args, "("..linkMatch..")%s+(%w+)");
		if not item or not player then
			self:Print("AddItem: Must supply an item link and a player name.");
		elseif not frame.selected then
			self:Print("AddItem: There is no raid selected");
		else
			if self:AddLootItemDB(frame.selected,item,player) then
				self:Print("AddItem: Added "..item);
			else
				self:Print("AddItem: Could not add "..item);
			end
		end
		KA_RaidTracker_Update();
		KA_RaidTracker_UpdateView();
	elseif command == "io" then
--		local checks = { "item:(%d+):","(%d+)%s?" };
		local checks = { "item:(%d+):" };
		for i,match in pairs(checks) do
			for itemid in string.gmatch(args,match) do
				itemid = tonumber(itemid);
				if db.ItemOptions[itemid] then
					self:Print(itemid.." is already in the Item Options list.");
				else
					db.ItemOptions[itemid] = {id = itemid};
					self:Print("Added "..itemid.." to the Item Options list.");
				end
			end
		end
		RT_ItemOptionsFrame:Show();
	elseif command == "options" or command == "o" then
		RaidTracker_OptionsFrame:Show();
	elseif command == "join" or command == "leave" then
		if not frame.selected then
			self:Print(command..": There is no raid selected");	return;
		end
		if args and strlen(args) > 0 then
			RT_JoinLeaveFrameNameEB:SetText(args);
		end
		RT_Buttons:ShowDialog("RT_JoinLeaveFrame", command, frame.selected);
	elseif command then
		self:Print("/rt - Shows the main window.");
		self:Print("/rt options|o - Shows Options window");
		self:Print("/rt io - Shows the ItemOptions window");
		self:Print("/rt io [ITEMLINK|ITEMID]... - Adds items to ItemOptions window");
		self:Print("/rt additem [ITEMLINK] [PLAYER] - Adds a loot item to the selected raid");
		self:Print("/rt join [PLAYER] - Add a player to the selected raid");
		self:Print("/rt leave [PLAYER] - Remmoves a player from the selected raid");
		self:Print("/rt deleteall - Deletes all raids");
		self:Print("/rt debug 1|0 - Enables/Disables debug mode");
		self:Print("/rt addwipe - Adds a Wipe with the current timestamp");
	else
		ShowUIPanel(frame);
	end
end


function KARaidTracker:OnEventCustom( frame, event, ... )
	local db = self._store.db;
	local options = self._options;
	local L = self._acelib.L;
	local LR = self._acelib.LR;

	local RaidLog = db.RaidLog;
	local julianne_died = false;

	-- delay initial events for a bit
	if ( RT_UpdateFrame.time and RT_UpdateFrame.time <= 2 ) then
		self:Debug("Queueing", frame and frame:GetName(), event, ...);
		tinsert(KA_RaidTracker_Events, {frame, event, ...} );
		return;
	end
	--self:Debug("Event", frame and frame:GetName(), event, ...);

	-- hacks for certiain bosses
	if event == "CHAT_MSG_MONSTER_YELL" or event == "CHAT_MSG_MONSTER_EMOTE" then -- Sorry, Majordomo needs a little hack
		if arg1 == L.BossKill_Yell_Majordomo then
			self:Debug("It's domo!");
			event = "CHAT_MSG_COMBAT_HOSTILE_DEATH";
			arg1 = string.gsub(self:ConvertGlobalString(UNITDIESOTHER), "%(%.%+%)", L["Majordomo Executus"]);
		elseif arg2 == "Karazhan - Chess, Victory Controller" then
			self:Print("Plz contact Eris at www.mldkp.net that Chess event logging is working (incl language)");
			event = "CHAT_MSG_COMBAT_HOSTILE_DEATH";
			arg1 = string.gsub(self:ConvertGlobalString(UNITDIESOTHER), "%(%.%+%)", L["Chess Event"]);
		elseif arg1 == L.BossKill_Yell_Chess_Event then
			self:Debug("Chess event");
			event = "CHAT_MSG_COMBAT_HOSTILE_DEATH";
			arg1 = string.gsub(self:ConvertGlobalString(UNITDIESOTHER), "%(%.%+%)", L["Chess Event"]);
		elseif arg1 == L.BossKill_Yell_Julianne then
			self:Debug("Romulo und Julianne");
			event = "CHAT_MSG_COMBAT_HOSTILE_DEATH";
			arg1 = string.gsub(self:ConvertGlobalString(UNITDIESOTHER), "%(%.%+%)", L["Julianne"]);
			julianne_died = true;
		end
	end

	if event == "RAID_ROSTER_UPDATE" or (event == "PARTY_MEMBERS_CHANGED" and GetNumRaidMembers() == 0) then
		local count = self:GetAttendeeCount();
		if count == 0 and options.CurrentRaid then
			self:EndRaidDB();									-- if a raid and no peoples, den mark them all lefted
		elseif count > 1 and not options.CurrentRaid then
			if self:CanAutoCreateLog() then
				self:CreateRaidDB();							-- if not a raid and peoples der and is okies to make raid by self, den makes a raid
			end
		elseif options.CurrentRaid then
			self:UpdateRaidDB();
		else
			return
		end
		KA_RaidTracker_Update();
		KA_RaidTracker_UpdateView();

	elseif event == "CHAT_MSG_LOOT" then
		local count = self:GetAttendeeCount();
		if count > 1 and not options.CurrentRaid then
			if self:CanAutoCreateLog() then
				self:CreateRaidDB();								-- if not a raid and peoples der and is okies to make raid by self, den makes a raid
			end
		end
		if options.CurrentRaid then
			local sPlayer, sLink, iCount, bFilter;					-- job here is to get player, link, item count, and if to engage item filters
			if		string.find(arg1, L.ReceivesLoot1) then
				_, _, sPlayer, sLink = string.find(arg1, L.ReceivesLoot1);
			elseif	string.find(arg1, L.ReceivesLoot2) then
				_, _, sLink = string.find(arg1, L.ReceivesLoot2);
				sPlayer = YOU;
			elseif	string.find(arg1, L.ReceivesLoot3) then
				_, _, sPlayer, sLink, iCount = string.find(arg1, L.ReceivesLoot3);
			elseif	string.find(arg1, L.ReceivesLoot4) then
				_, _, sLink, iCount = string.find(arg1, L.ReceivesLoot4);
				sPlayer = YOU;
			end
			if sPlayer and sPlayer == YOU then
				sPlayer = UnitName("player");
			end
			self:Debug("CHAT_MSG_LOOT", "arg1",arg1, "results",sPlayer,sLink,iCount);
			if not sLink or not sPlayer then return; end

			self:AddLootItemDB( options.CurrentRaid, sLink, sPlayer, iCount );
		end
		KA_RaidTracker_Update();
		KA_RaidTracker_UpdateView();

	elseif event == "UPDATE_MOUSEOVER_UNIT" then
		if not options.AutoBoss == 1 then return end

		local autoboss_unitname = UnitName("mouseover");
		local autoboss_newboss;
		if not UnitIsFriend("mouseover", "player") and not UnitInRaid("mouseover") and not UnitInParty("mouseover") then
			--self:Debug("possible mouseover unit update", autoboss_unitname);
			if RaidTracker_BossUnitTriggers[autoboss_unitname] then
				if RaidTracker_BossUnitTriggers[autoboss_unitname] ~= "IGNORE" then
					autoboss_newboss = RaidTracker_BossUnitTriggers[autoboss_unitname];
					KA_RaidTracker_AutoBossChangedTime = GetTime();
				end
			elseif RaidTracker_BossUnitTriggers["DEFAULTBOSS"] and (options["BossCooldown"] == 0 or (GetTime() > (KA_RaidTracker_AutoBossChangedTime + options["BossCooldown"]))) then
				autoboss_newboss = RaidTracker_BossUnitTriggers["DEFAULTBOSS"];
			else
				autoboss_newboss = nil
			end
			if autoboss_newboss and options.BossName ~= autoboss_newboss then
				options.BossName = autoboss_newboss;
				self:Print("RaidTracker AutoBoss Update: "..autoboss_newboss);
			end
		end
	end

	-- end of events that dont require current raid
	if not options.CurrentRaid then
		return

	elseif event == "PLAYER_ENTERING_WORLD" then
		self:UpdateRaidDB();
		KA_RaidTracker_UpdateView();
		KA_RaidTracker_Update();

	elseif event == "CHAT_MSG_SYSTEM" then
		local name = UnitName("player");
		if name and name ~= UKNOWNBEING and name ~= UNKNOWN then
			local sDate, sPlayer = self:GetDisplayTime();
			if string.find(arg1, L.LeftRaid) then
				_, _, sPlayer = string.find(arg1, L.LeftRaid);
			else 
				_, _, sPlayer = string.find(arg1, L.LeftParty);
			end
			if sPlayer and sPlayer ~= name and sPlayer ~= UKNOWNBEING and sPlayer ~= UNKNOWN and db.Online[sPlayer] then
				tinsert(RaidLog[options.CurrentRaid].Leave,	{ player = sPlayer, time = sDate, }	);
				db.Online[sPlayer] = nil;
				self:Debug(sPlayer, "LEFT", sDate);
			end

			KA_RaidTracker_UpdateView();
			KA_RaidTracker_Update();
		end

	elseif ( event == "CHAT_MSG_COMBAT_HOSTILE_DEATH" ) then
		self:Debug("CHAT_MSG_COMBAT_HOSTILE_DEATH",arg1);
		local bosskilled, autoboss_newboss;
		local sDate = self:GetDisplayTime();

		for unit in string.gmatch(arg1, self:ConvertGlobalString(UNITDIESOTHER)) do
			self:Debug("CHAT_MSG_COMBAT_HOSTILE_DEATH","unit", unit);
			if options.AutoBoss == 2 then
				if(not db.Online[unit]) then
					if(RaidTracker_BossUnitTriggers[unit]) then
						if(RaidTracker_BossUnitTriggers[unit] ~= "IGNORE") then
							autoboss_newboss = RaidTracker_BossUnitTriggers[unit];
						end
					elseif(RaidTracker_BossUnitTriggers["DEFAULTBOSS"]) then
						autoboss_newboss = RaidTracker_BossUnitTriggers["DEFAULTBOSS"];
					else
						autoboss_newboss = nil
					end
					if(autoboss_newboss and options.BossName ~= autoboss_newboss) then
						options.BossName = autoboss_newboss;
						if (unit == "Hase") then
							unit = "V\195\182lli hat den alten Hoppel Hasen erledigt!"; --Just a Joke
						end
						self:Print("KA_RaidTracker AutoBoss Update: "..autoboss_newboss.." ("..unit..")", 1, 1, 0);
					end
				end
			end

			if RaidTracker_BossUnitTriggers[unit] and RaidTracker_BossUnitTriggers[unit] ~= "IGNORE" then
				local newboss = 1;
				-- Romulo and Julianne Hack
				if RaidTracker_BossUnitTriggers[unit] == "Romulo and Julianne" then
					if (julianne_died == false) then
						return;
					end;
				end
				-- Romulo and Julianne Hack
				bosskilled = RaidTracker_BossUnitTriggers[unit];
				if not RaidLog[options.CurrentRaid].BossKills then
					RaidLog[options.CurrentRaid].BossKills = { };
				end
				-- is the boss already killed?
				for key, val in pairs(RaidLog[options.CurrentRaid].BossKills) do
					if val.boss == bosskilled then
						newboss = 0;
					end
				end
				if newboss == 1 then
					tinsert( RaidLog[options.CurrentRaid].BossKills,
						{
							boss = bosskilled,
							time = sDate,
							attendees = (options.LogAttendees==1) or (options.LogAttendees==3) and self:GetAttendees(true, options.LogAttendees == 3) or { };
						}
					);
					self:Print("RaidTracker Boss Kills: Set kill time for \""..bosskilled.."\" to "..sDate, 1, 1, 0);
					if options.BossAsk == 1 then
						RT_BossNextFrame:Show();
					end;
					if options.LogGuild == 1 then
						KA_RaidTrackerAddGuild();
					end;
				end
				--if(not RaidLog[options.CurrentRaid].BossKills[bosskilled]) then
				--	RaidLog[options.CurrentRaid].BossKills[bosskilled] = sDate;
				--	self:Print("KA_RaidTracker Boss Kills: Set kill time for \""..bosskilled.."\" to "..sDate, 1, 1, 0);
				--end
			end
		end

	elseif event == "UNIT_HEALTH" then
		if options.WipeAsk == 0 or InCombatLockdown() or GetTime()<KA_RaidTracker_LastWipe then return end;

		local memberdead, count = 0, self:GetAttendeeCount();
		for i = 1, count, 1 do
			if UnitIsDeadOrGhost(self:GetAttendeeUnitId(i)) then
				memberdead = memberdead + 1;
			end
		end
		if memberdead == count then
			KA_RaidTracker_AddWipe();
			RT_AcceptWipeFrame:Hide();
		elseif (memberdead / count) > options.WipePercent then
			RT_AcceptWipeFrame:Show();
		end;

	elseif event == "ZONE_CHANGED_NEW_AREA" then
		if options.AutoZone == 1 then
			self:DoZoneCheck();
			KA_RaidTracker_DoRaidIdCheck();
		end
		KA_RaidTracker_Update();
		KA_RaidTracker_UpdateView();

	elseif ( event == "UPDATE_INSTANCE_INFO" ) then
		KA_RaidTracker_DoRaidIdCheck();
	end

end


-- unifies attendees info lookup and raid/party/bgs/arenas

function KARaidTracker:GetAttendeesType( )
	if IsActiveBattlefieldArena() == 1 then return "Arena" end
	if GetNumBattlefieldScores() > 0 then return "Battlegroup" end
	if GetNumRaidMembers() > 0 then return "Raid" end
	if GetNumPartyMembers() > 0 then return "Party" end
	return "Solo"
end

function KARaidTracker:CanAutoCreateLog( )
	local o, t = self._options, self:GetAttendeesType();
	return (t=="Arena" and o.AutoAreana==2) or (t=="Battlegroup" and o.AutoBattlegroup==2) or
		   (t=="Raid" and o.AutoRaid==2) or (t=="Party" and o.AutoParty==2) or (o.AutoSolo==2);
end

function KARaidTracker:GetAttendeeCount( )		-- if party logging is off then will return 0 if on a party
	local o, t = self._options, self:GetAttendeesType();
	local options = self._options;
	if GetNumRaidMembers() > 0 and (options.AutoRaid >= 1 or
		(GetNumBattlefieldScores() > 0 and options.AutoBattlegroup >= 1)) then
		return GetNumRaidMembers()
	end
	if GetNumPartyMembers() > 0 and (options.AutoParty >= 1 or
		(IsActiveBattlefieldArena() == 1 and options.AutoAreana >= 1)) then
		return GetNumPartyMembers() + 1
	end
	if options.AutoSolo >= 1 then return 1 end
	return 0;
end

function KARaidTracker:GetAttendeeUnitId( i )
	if tonumber(i)==nil then return i end
	if GetNumRaidMembers() > 0 then return "raid"..i end
	return (i > GetNumPartyMembers()) and "player" or ("party"..i);
end

function KARaidTracker:GetAttendeeInfo( i, isExtended )
	local unit = self:GetAttendeeUnitId(i);

	local name, online, zone, level, class;
	if GetNumRaidMembers() > 0 and tonumber(i) then
		name,_,_,level,_,class,zone,online = GetRaidRosterInfo(i);	-- name, rank, subgroup, level, class, fileName, zone, online = GetRaidRosterInfo(i);
	else
		name,online,zone = UnitName(unit),UnitIsConnected(unit),self:UnitZone(unit);
	end
	if not isExtended then
		return name, online, zone;
	end

	level = level or UnitLevel(unit);
	if not class then _, class = UnitClass(unit); end
	local sex, guild, _, race = UnitSex(unit), GetGuildInfo(unit), UnitRace(unit);
	return name, online, zone, level, class, sex, guild, race;
end

function KARaidTracker:GetAttendees( chkOnline, chkZone )
	local options = self._options;
	local count, t = self:GetAttendeeCount(), { };
	if count == 0 then return t end

	for i = 1, count do
		name, online, zone = self:GetAttendeeInfo(i);
		if (name and name ~= UKNOWNBEING and name ~= UNKNOWN) and 		-- check name and itentity
			(not chkZone or not zone or zone==GetRealZoneText()) and	-- should check, haz a zone, zone matches
			(not chkOnline or online) then  							-- should check, is online
				tinsert(t, i);
		end
	end

	return t;
end


function KARaidTracker:SortRaidTable()
	local db = self._store.db;
	local func = function( a1, a2 )
			return a1 and a2 and self:GetTimestamp(a1.key) > self:GetTimestamp(a2.key);
		end
	table.sort( db.RaidLog, func );
end

function KARaidTracker:UpdateRaidDB( isNew )
	local db = self._store.db;
	local options = self._options;
	if not options.CurrentRaid then return end
	local t = db.RaidLog[options.CurrentRaid];

	-- update player info, join and leave times
	local tAttend, sDate = self:GetAttendees(), self:GetDisplayTime();
	for i, v in pairs(tAttend) do
		local name, online, zone, level, class, sex, guild, race = self:GetAttendeeInfo(v, true);

		if not t.PlayerInfos[name] then
			t.PlayerInfos[name] = { };
		end
		local tInfo = t.PlayerInfos[name];
		if options.SavePlayerDetail == 1 then
			if race then tInfo.race = race; end
			if class then tInfo.class = class; end
			if sex then tInfo.sex = sex; end
			if level > 0 then tInfo.level = level; end
			if guild then tInfo.guild = guild; end
		end

		if isNew or online ~= db.Online[name] then						-- this will change status for existing player
			if isNew or online then
				tinsert( t.Join,  { player = name, time = sDate } );
			end
			if not online then
				tinsert( t.Leave, {	player = name, time = sDate } );
			end
		end

		db.Online[name] = online;
	end

	if options.AutoZone == 1 then
		self:DoZoneCheck();
--		KA_RaidTracker_DoRaidIdCheck();
	end

end

function KARaidTracker:CreateRaidDB( )
	local db = self._store.db;
	local options = self._options;
	KA_RaidTracker_GetGameTimeOffset();
	local sDate = self:GetDisplayTime();

	-- start new raid
	self:EndRaidDB()
	tinsert( db.RaidLog, 1, {
		Loot = { },
		Join = { },
		Leave = { },
		PlayerInfos = { },
		BossKills = { },
		key = sDate
	} );
	options.CurrentRaid = 1;
	self:SortRaidTable();						-- sort by creation timestamp
	self:UpdateRaidDB( true );					-- setup player info, join and leave times

	self:Print("Joined new raid at " .. sDate);
end

function KARaidTracker:DeleteRaidDB( raidid )
	local frame = RaidTrackerFrame;
	local db = self._store.db;
	local options = self._options;

	if frame.selected == raidid then			-- unhook frame if selected raid
		if frame.selected > 1 then
			 frame.selected = frame.selected - 1;
		end
		frame.type = "raids";
	end
	if options.CurrentRaid == raidid then		-- unhook db if current raid
		options.CurrentRaid = nil;
		db.Online = { };
	end

	table.remove(db.RaidLog, raidid);
end

function KARaidTracker:EndRaidDB()
	local db = self._store.db;
	local options = self._options;
	
	if options.CurrentRaid then
		local raid = db.RaidLog[options.CurrentRaid];
		local sTime = self:GetDisplayTime();
		self:Print("Ending current raid at "..sTime);
		
		for k, v in pairs(db.Online) do
			tinsert( raid.Leave, { player = k, time = sTime, } );
		end
		if not raid.End then
			raid.End = sTime;
		end
		options.CurrentRaid = nil;
	end

	db.Online = { };
end

function KARaidTracker:SnapshotRaidDB( )
	local db = self._store.db;
	local options = self._options;
	if not options.CurrentRaid then return end

	self:Print("Snapshotting current raid.");
	local raid = db.RaidLog[options.CurrentRaid];
	local sDate = self:GetDisplayTime();
	local t = {
		Loot = { },
		Join = { },
		Leave = { },
		PlayerInfos = { },
		BossKills = { },
		key = sDate,
		End = sDate,
	};
	t.zone = raid.zone;
	for k, v in pairs(db.Online) do
		t.PlayerInfos[k] = { };
		tinsert(t.Join, { player = k, time = sDate } );
		tinsert(t.Leave, { player = k, time = sDate } );
	end
	tinsert(db.RaidLog, 2, t);
end



function KARaidTracker:CreateLootItemDB( frame, sLink, sPlayer, iCount, ioGetCost )
	local options = self._options;
	local sColor, sItem, sName = self:GetLinkMeta(sLink);
	if not sItem then return end
	local sNote, sCosts;

	if ioGetCost then												-- get cost from another system
		local _, _, s0, s1, s2, s3 = string.find(sItem, "^(%d+):(%d+):(%d+):(%d+)");
		self:Debug("Splitted", s0, s1, s2, s3);
		if		DKPValues and DKPValues[tostring(s0)] then			-- AdvancedItemTooltip
			sCosts = tonumber(DKPValues[tostring(s0)]);
		elseif	HDKP_GetDKP then									-- HoB_DKP
			sCosts = tonumber(HDKP_GetDKP(s0, s1, s2, s3));
		end
		if sCosts == 0 then sCosts = nil end;
	end

	local name, link, quality, iLevel, minLevel, class, subclass, maxStack, invtype, icon = GetItemInfo("item:"..sItem);
	if icon then
		_, _, icon = string.find(icon, "^.*\\(.*)$");
	end
	local lootItem = {
		player = sPlayer,
		item = {
			c = sColor,
			id = sItem,
			tooltip = (options.SaveTooltips == 1) and self:GetItemTooltip(sItem) or { },
			name = sName,
			icon = icon,
			count = iCount,
			class = class,
			subclass = subclass,
		},
		costs = sCosts,
		boss = (options.AutoBoss >= 1) and options.BossName or nil,
		note = sNote,
		zone = GetRealZoneText(),
		time = self:GetDisplayTime(),
		attendees = (options.LogAttendees == 2) and self:GetAttendees(true) or { },
	}
	return lootItem;
end

function KARaidTracker:AddLootItemDB( raidid, sLink, sPlayer, iCount )
	self:Debug("AddLootItemDB","rid",raidid,"link",sLink,"player",sPlayer)
	local frame = RaidTrackerFrame;
	local db = self._store.db;
	local options = self._options;
	if not raidid then return end
	local items = db.RaidLog[raidid].Loot;

	iCount = tonumber(iCount) or 1;

	-- check via Options
	local sColor, sItem, sName = self:GetLinkMeta(sLink);
	local rarity = self.ColorToRarityTable[sColor];
	local _, _, itemid = string.find(sItem, "^(%d+):%d+:%d+:%d+");
	local io = db.ItemOptions[tonumber(itemid)];
	local ioGroup, ioCostAsk, ioGetCost;
	if (io and io.Track==0)	or ((not io or not io.Track) and not
		(rarity >= options.LogItems) ) then return; end
	ioGroup = (io and io.Group==1) or ((not io or not io.Group) and
		(options.GroupItems~=0 and rarity <= options.GroupItems) );
	ioGetCost = (io and io.CostGet==1) or ((not io or not io.CostGet) and
		(options.CostGet~=0 and rarity >= options.CostGet) );
	ioCostAsk = (io and io.CostAsk==1) or ((not io or not io.CostAsk) and
		(options.CostAsk~=0 and rarity >= options.CostAsk) );

	-- add loot item
	local t;
	if ioGroup then
		for k, v in pairs(items) do
			if ( v.item.name == sName and v.player == sPlayer ) then
				v.item.count = (v.item.count or 0) + iCount;
				t = v;
				break;
			end
		end
	end
	if not t then
		t = self:CreateLootItemDB(frame, sLink, sPlayer, iCount, ioGetCost);
		t.id = #items + 1
		tinsert(items, t.id, t);
		if ioCostAsk then									-- code and idea from tlund
			RT_Buttons:ShowDialog("RT_EditCostFrame", "itemcost", raidid, 1);
		end
	end
	return t;
end

function KARaidTracker:GetLinkMeta( sLink )
	if not sLink then return end
	local _, _, sColor, sItemStr, sName = string.find(sLink, "|c(%x+)|Hitem:([-%d:]+)|h%[(.-)%]|h|r");
	return sColor, sItemStr, sName;
end

function KARaidTracker:GetDisplayTime( )
	return self:GetTimeDisplay(nil, self._options.Timezone, KA_RaidTracker_TimeOffset);
end

function KARaidTracker:DoZoneCheck( )
	local db = self._store.db;
	local options = self._options;
	local LR = self._acelib.LR;

	if not options.CurrentRaid then return end

	self:Debug("DoZoneCheck - real",GetRealZoneText(),LR[GetRealZoneText()],db.ZoneTriggers[LR[GetRealZoneText()]]);
	self:Debug("DoZoneCheck - zone",GetZoneText(),LR[GetZoneText()],db.ZoneTriggers[LR[GetZoneText()]]);
	self:Debug("DoZoneCheck - sub",GetSubZoneText(),LR[GetSubZoneText()],db.ZoneTriggers[LR[GetSubZoneText()]]);

	if not db.RaidLog[options.CurrentRaid].zone then
		local zone = db.ZoneTriggers[LR[GetRealZoneText()]];
		if zone then
			self:Print("Setting current raid zone to "..GetRealZoneText());
			db.RaidLog[options.CurrentRaid].zone = LR[GetRealZoneText()];
		end
	end
	return true;
end


--[[ *****************************************************************************************************
	******************************************************************************************************
	******************************************************************************************************
	****************************************************************************************************** ]]


local KA_RaidTracker_RarityTable = {
	["ff9d9d9d"] = 1,
	["ffffffff"] = 2,
	["ff1eff00"] = 3,
	["ff0070dd"] = 4,
	["ffa335ee"] = 5,
	["ffff8000"] = 6,
	["ffe6cc80"] = 7,
};



function KA_RaidTracker_GetRaidTitle(raidid, hideid, showzone, shortdate)
	local raidLog = KARaidTrackerDB.RaidLog
	local RaidTitle = "";
	if ( raidLog[raidid] and raidLog[raidid].key ) then
		local _, _, mon, day, year, hr, min, sec = string.find(raidLog[raidid].key, "(%d+)/(%d+)/(%d+) (%d+):(%d+):(%d+)");
		if ( mon ) then
			local months = {
				"Jan", "Feb", "Mar", "Apr", "May", "Jun", "Jul", "Aug", "Sep", "Oct", "Nov", "Dec"
			};
			if ( not hideid ) then
				RaidTitle = RaidTitle .. "[" .. (getn(raidLog)-raidid+1) .. "] ";
			end
			if ( not shortdate ) then
				RaidTitle = RaidTitle .. months[tonumber(mon)] .. " " .. day .. " '" .. year .. ", " .. hr .. ":" .. min;
			else
				RaidTitle = RaidTitle .. mon .. "/" .. day .. " " .. hr .. ":" .. min;
			end
			if showzone and raidLog[raidid].zone then
				RaidTitle = RaidTitle .. " " .. KARaidTracker._acelib.L[raidLog[raidid].zone];
			end
			return RaidTitle;
		else
			return "";
		end
	end
	return "";
end


function KA_RaidTracker_GetLootId(raidid, sPlayer, sItem, sTime)
	for k, v in pairs( KARaidTrackerDB.RaidLog[raidid].Loot ) do
		if v.player == sPlayer and v.item.id == sItem and v.time == sTime then
			return k;
		end
	end
end

function KA_RaidTracker_Update(frame)
	local frame = RaidTrackerFrame;
	local db = KARaidTrackerDB;

	-- main frame top buttons
	if db.Options.CurrentRaid then
		RaidTrackerFrameEndRaidButton:Enable();
		RaidTrackerFrameSnapshotButton:Enable();
	else
		RaidTrackerFrameEndRaidButton:Disable();
		RaidTrackerFrameSnapshotButton:Disable();
	end
	if KARaidTracker:GetAttendeeCount() > 0 then
		RaidTrackerFrameNewRaidButton:Enable();
	else
		RaidTrackerFrameNewRaidButton:Disable();
	end;
	if #RT_Buttons.LastPage > 0 then
		RaidTrackerFrameBackButton:Enable();
	else
		RaidTrackerFrameBackButton:Disable();
	end

	-- process
	local numEntries = getn(db.RaidLog);
	if numEntries <= 0 then
		frame.type = "raids";
		frame.selected = nil;
		RaidTrackerFrameEmptyFrame:Show();
		RT_DetailFramePlayers:Hide();
		RT_DetailFrameItems:Hide();
		RT_DetailFrameRaids:Hide();
		RT_DetailFrameEvents:Hide();
		RaidTrackerFrameDeleteButton:Disable();
		RaidTrackerFrameViewButton:Disable();
	else
		RaidTrackerFrameEmptyFrame:Hide();
		RaidTrackerFrameDeleteButton:Enable();
		frame.type = frame.type or "raids";

		local type = frame.type;
		if type == "raids" or type == "items" then
			frame.selected = frame.selected or 1;
			if not db.RaidLog[frame.selected] then
				frame.selected = numEntries;
			end
			if getn(db.RaidLog[frame.selected].Loot) == 0 then
				RaidTrackerFrameViewButton:Disable();
			else
				RaidTrackerFrameViewButton:Enable();
			end
			RaidTrackerFrameViewButton:SetText( (type == "raids" and "View Items") or "View Players");
		elseif type == "player" or type == "playeritems" then
			local hasItem;
			for k, v in pairs(db.RaidLog) do
				local loot = v.Loot;
				for key, val in pairs(loot) do
					if val.player == frame.player then hasItem = 1;	break; end
				end
				if hasItem then	break; end
			end
			if not hasItem then
				if type == "playeritems" then
					frame.type = "player";
					KA_RaidTracker_Update();
					KA_RaidTracker_UpdateView();
					return;
				end
				RaidTrackerFrameViewButton:Disable();
			else
				RaidTrackerFrameViewButton:Enable();
			end
			RaidTrackerFrameDeleteButton:Disable();
			RaidTrackerFrameViewButton:SetText((type == "player" and "View Loot") or "View Raids" );
		elseif type == "itemhistory" or type == "events" then
			RaidTrackerFrameViewButton:Disable();
			RaidTrackerFrameDeleteButton:Disable();
		end
	end

	-- ScrollFrame update
	local numEntries = getn(db.RaidLog);
	FauxScrollFrame_Update(RT_ListScrollFrame, numEntries, 6, 16, nil, nil, nil, RT_ListHighlightFrame, 293, 316 );
	RT_ListHighlightFrame:Hide();
	for i=1, 6, 1 do
		local line = getglobal("RT_RaidLineFrame" .. i);
		local raidid = i + FauxScrollFrame_GetOffset(RT_ListScrollFrame);

		if raidid > numEntries then
			line.raidid = nil;
			line:Hide();
		else
			local raidTitle = KA_RaidTracker_GetRaidTitle(raidid, nil, 1, 1);
			local raidTag = db.RaidLog[raidid].note;
			line.raidid = raidid;
			line:SetText( (raidTitle or "") .. ((raidTag and " (" .. raidTag .. ")") or "") );
			line:Show();

			-- Place the highlight and lock the highlight state
			if ( frame.selected and frame.selected == raidid ) then
				KA_RaidTrackerSkillHighlight:SetVertexColor(1, 1, 0);
				RT_ListHighlightFrame:SetPoint("TOPLEFT", "RT_RaidLineFrame"..i, "TOPLEFT", 0, 0);
				RT_ListHighlightFrame:Show();
				line:LockHighlight();
			else
				line:UnlockHighlight();
			end
		end
	end

	-- build status text
	local self = KARaidTracker;
	local c, cOL, cZ = self:GetAttendeeCount(), #self:GetAttendees(true), #self:GetAttendees(true, true);
	local sc =  cOL .. ((cOL~=c) and ("/" .. c) or "");
	local sz = GetRealZoneText() .. (GetSubZoneText()~="" and (": "..GetSubZoneText()) or "");
	local szc =  (cZ>0 and cZ~=cOL) and ("("..cZ..")") or "";
	local text = self:GetAttendeesType() .. " (" .. sc  .. ") - " .. sz .. " " .. szc;
	RaidTrackerFrameHeaderText:SetText( text );
end

function KA_RaidTracker_SelectRaid(id)
	local raidid = id + FauxScrollFrame_GetOffset(RT_ListScrollFrame);
	KA_RaidTracker_GetPage();
	RaidTrackerFrame.selected = raidid;
	--if ( getn(KARaidTrackerDB.RaidLog[raidid].Loot) == 0 or ( RaidTrackerFrame.type and RaidTrackerFrame.type ~= "items" ) ) then
		RaidTrackerFrame.type = "raids";
	--end

	KA_RaidTracker_UpdateView();
	KA_RaidTracker_Update();
end

function KA_RaidTracker_ShowInfo( player )
	KA_RaidTracker_GetPage();

	RaidTrackerFrame.type = "player";
	RaidTrackerFrame.player = player;
	RaidTrackerFrame.selected = nil;

	KA_RaidTracker_Update();
	KA_RaidTracker_UpdateView();
end


function KA_RaidTracker_Sort(tbl, method, way)
	if ( way == "asc" ) then
		table.sort(	tbl,
			function(a1, a2)
				return a1[method] < a2[method];
			end
		);
	else
		table.sort(	tbl,
			function(a1, a2)
				return a1[method] > a2[method];
			end
		);
	end
	return tbl;
end


function KA_RaidTracker_SortPlayerRaids(id)
	local sortOptions = KARaidTrackerDB.SortOptions
	if ( RaidTrackerFrame.type == "itemhistory" ) then
		local table = { "name", "looter" };

		if ( sortOptions["itemhistorymethod"] == table[id] ) then
			if ( sortOptions["itemhistoryway"] == "asc" ) then
				sortOptions["itemhistoryway"] = "desc";
			else
				sortOptions["itemhistoryway"] = "asc";
			end
		else
			sortOptions["itemhistoryway"] = "asc";
			sortOptions["itemhistorymethod"] = table[id];
		end
	else
		if ( sortOptions["playerraidway"] == "asc" ) then
			sortOptions["playerraidway"] = "desc";
		else
			sortOptions["playerraidway"] = "asc";
		end
	end
	KA_RaidTracker_UpdateView();
end


function KA_RaidTracker_CompareItems(a1, a2)
	-- This function could probably be better, but I can't think of any better way while still maintaining functionality
	local util = KarmaUtility
	local sortOptions = KARaidTrackerDB.SortOptions

	local filter, method, way = sortOptions["itemsfilter"], sortOptions["itemmethod"], sortOptions["itemway"];
	if ( RaidTrackerFrame.type == "playeritems" ) then
		filter, method, way = sortOptions["playeritemsfilter"], sortOptions["playeritemmethod"], sortOptions["playeritemway"];
	end

	-- Check to see if it matches the rarity requirements
	if ( KA_RaidTracker_RarityTable[a1.item.c] < filter ) then
		return false;
	elseif ( KA_RaidTracker_RarityTable[a2.item.c] < filter ) then
		return true;
	end

	if ( method == "name" ) then
		local c1, c2 = a1.item.name, a2.item.name;
		if ( c1 == c2 ) then
			c1, c2 = a1.player, a2.player;
		end
		if ( way == "asc" ) then
			return c1 < c2;
		else
			return c1 > c2;
		end
	elseif ( method == "looter" ) then
		local c1, c2 = a1.player, a2.player;
		if ( c1 == c2 ) then
			c1, c2 = KA_RaidTracker_RarityTable[a2.item.c], KA_RaidTracker_RarityTable[a1.item.c];
			if ( c1 == c2 ) then
				c1, c2 = a1.item.name, a2.item.name;
			end
		end
		if ( way == "asc" ) then
			return c1 < c2;
		else
			return c1 > c2;
		end
	elseif ( method == "looted" ) then
		if ( way == "asc" ) then
			return util:GetTimestamp(a1.time) < util:GetTimestamp(a2.time);
		else
			return util:GetTimestamp(a1.time) > util:GetTimestamp(a2.time);
		end
	else
		local c1, c2 = KA_RaidTracker_RarityTable[a1.item.c], KA_RaidTracker_RarityTable[a2.item.c];
		if ( c1 == c2 ) then
			c1, c2 = a1.item.name, a2.item.name;
			if ( c1 == c2 ) then
				c1, c2 = a1.player, a2.player;
			else
				return c1 < c2;
			end
		end
		if ( way == "asc" ) then
			return c1 < c2;
		else
			return c1 > c2;
		end
	end
end


function KA_RaidTracker_SortItemBy(id)
	local sortOptions = KARaidTrackerDB.SortOptions
	local table = { "name", "looted", "looter", "rarity" };
	local prefix = "";
	if ( RaidTrackerFrame.type == "playeritems" ) then
		prefix = "player";
	end
	if ( sortOptions[prefix.."itemmethod"] == table[id] ) then
		if ( sortOptions[prefix.."itemway"] == "asc" ) then
			sortOptions[prefix.."itemway"] = "desc";
		else
			sortOptions[prefix.."itemway"] = "asc";
		end
	else
		sortOptions[prefix.."itemmethod"] = table[id];
		sortOptions[prefix.."itemway"] = "asc";
	end
	KA_RaidTracker_UpdateView();
end

function KA_RaidTracker_SortBy(id)
	local sortOptions = KARaidTrackerDB.SortOptions
	local table = { "name", "join", "leave" };
	if ( sortOptions["method"] == table[id] ) then
		if ( sortOptions["way"] == "asc" ) then
			sortOptions["way"] = "desc";
		else
			sortOptions["way"] = "asc";
		end
	else
		sortOptions["method"] = table[id];
		if ( table[id] ~= "leave" ) then
			sortOptions["way"] = "asc";
		else
			sortOptions["way"] = "desc";
		end
	end
	KA_RaidTracker_UpdateView();
end

function KA_RaidTracker_UpdateView( )
	local frame = RaidTrackerFrame;
	local db = KARaidTrackerDB;

	if RaidTrackerFrameEmptyFrame:IsVisible() then return end
	local raidid = frame.selected;
	local raid = db.RaidLog[raidid]
	if raid == nil then
		raidid = nil;
	end

	if frame.type == "events" then
		RaidTrackerFrameView2Button:SetText("View Raid");
	else
		RaidTrackerFrameView2Button:SetText("View Events");
		if not raidid or ( (not raid.BossKills or getn(raid.BossKills) == 0) and (not raid.Events or getn(raid.Events) == 0)) then
			RaidTrackerFrameView2Button:Disable();
		else
			RaidTrackerFrameView2Button:Enable();
		end
	end

	if ( frame.type == "raids" or not frame.type ) then
		RT_DetailFramePlayers:Show();
		RT_DetailFrameRaids:Hide();
		RT_DetailFrameItems:Hide();
		RT_DetailFrameEvents:Hide();
		local players = { };
		if ( raid ) then

			local playerIndexes = { };
			for key, val in pairs(raid.Join) do
				if ( val.player ) then
					local id = playerIndexes[val.player];
					local time = KarmaUtility:GetTimestamp(val.time);
					if ( not id or time < players[id]["join"] ) then

						if ( playerIndexes[val.player] ) then
							players[id] = {
								["join"] = time,
								["name"] = val.player
							};
						else
							tinsert(players, {
								["join"] = time,
								["name"] = val.player
							});
							playerIndexes[val.player] = getn(players);
						end
					end
					id = playerIndexes[val.player];
					if ( not players[id]["lastjoin"] or players[id]["lastjoin"] < time ) then
						players[id]["lastjoin"] = time;
					end
				end
			end
			for key, val in pairs(raid.Leave) do
				local id = playerIndexes[val.player];
				local time = KarmaUtility:GetTimestamp(val.time);
				if ( id ) then
					if ( ( not players[id]["leave"] or time > players[id]["leave"] ) and time >= players[id]["lastjoin"] ) then
						players[id]["leave"] = time;
					end
				end
			end
			for k, v in pairs(players) do
				if ( not v["leave"] ) then
					-- Very ugly hack, I know :(
					players[k]["leave"] = 99999999999;
				end
			end
			players = KA_RaidTracker_Sort(players, db.SortOptions["method"], db.SortOptions["way"]);
			RT_DetailFramePlayers.raidid = raidid;
			RT_DetailFramePlayers.players = players;
			KA_RaidTracker_DetailScrollFramePlayers_Update();
		end
		RT_DetailFramePlayersText:SetText("Participants (" .. getn(players) .. ")");
		RT_DetailFramePlayers:Show();
	elseif ( frame.type == "items" ) then
		RT_DetailFramePlayers:Hide();
		RT_DetailFrameRaids:Hide();
		RT_DetailFrameItems:Show();
		RT_DetailFrameEvents:Hide();
		local numItems, numHidden = 0, 0;
		if raid then
			local keystoremove = {};
			local loot = {}
			for k, v in pairs(raid.Loot) do
				loot[k] = v;
			end
			table.sort( loot, KA_RaidTracker_CompareItems );
			for key, val in pairs(loot) do
--				if (not val.item.tooltip or (getn(val.item.tooltip) == 0)) and db.Options.SaveTooltips == 1
--						and GetItemInfo("item:" .. val.item.id) then
--					val.item.tooltip = KARaidTracker:GetItemTooltip(val.item.id);
--				end
				if KA_RaidTracker_RarityTable[val.item.c] >= db.SortOptions.itemsfilter then
					numItems = numItems + 1;
				else
					tinsert(keystoremove, key);
					numHidden = numHidden + 1;
				end
			end
			for key, val in pairs(keystoremove) do
				table.remove(loot, val);
			end
			RT_DetailFrameItems.raidid = raidid;
			RT_DetailFrameItems.loot = loot;

			KA_RaidTracker_DetailScrollFrameItems_Update();
			RT_DetailFrameItems:Show();
		end
		if ( numHidden == 0 ) then
			RT_DetailFrameItemsText:SetText("Items ("..numItems.."):");
		else
			RT_DetailFrameItemsText:SetText("Items ("..numItems.."/"..numHidden + numItems..")");
		end
		RT_RarityDropdown:OnUpdate( );

	elseif ( frame.type == "player" ) then
		RT_DetailFramePlayers:Hide();
		RT_DetailFrameRaids:Show();
		RT_DetailFrameItems:Hide();
		RT_DetailFrameEvents:Hide();
		RT_DetailFrameRaidsTabLooter:Hide();
		RT_DetailFrameRaidsTab1:SetWidth(300);
		RT_DetailFrameRaidsTab1Middle:SetWidth(290);
		local name = frame.player;

		local raids = { };
		for k, v in pairs(db.RaidLog) do
			local isInRaid;
			for key, val in pairs(v.Join) do
				if ( val.player == name ) then
					tinsert(raids, { k, v });
					break;
				end
			end
		end

		table.sort(
			raids,
			function(a1, a2)
				if ( db.SortOptions["playerraidway"] == "asc" ) then
					return KarmaUtility:GetTimestamp(a1[2].key) < KarmaUtility:GetTimestamp(a2[2].key);
				else
					return KarmaUtility:GetTimestamp(a1[2].key) > KarmaUtility:GetTimestamp(a2[2].key);
				end
			end
		);
		RT_DetailFrameRaids.data = raids;
		RT_DetailFrameRaids.name = name;
		RT_DetailFrameRaids.maxlines = #raids;
		KA_RaidTracker_DetailScrollFramePlayer_Update();
		RT_DetailFrameRaidsText:SetText(name .. "'s Raids (" .. #raids .. "):");
		RT_DetailFrameRaids:Show();

	elseif ( frame.type == "itemhistory" ) then
		RT_DetailFramePlayers:Hide();
		RT_DetailFrameRaids:Show();
		RT_DetailFrameItems:Hide();
		RT_DetailFrameEvents:Hide();
		RT_DetailFrameRaidsTabLooter:Show();
		RT_DetailFrameRaidsTab1:SetWidth(163);
		RT_DetailFrameRaidsTab1Middle:SetWidth(155);

		local name, totalItems = frame.itemname, 0;
		local loot = { };
		for k, v in pairs(db.RaidLog) do
			for k1, v1 in pairs(v.Loot) do
				if v1.item.name == name then
					tinsert(loot, { k, v, v1 });
					totalItems = totalItems + (v1.item.count or 1);
				end
			end
		end
		local so, rt = db.SortOptions, KARaidTracker;
		table.sort( loot, function(a, b)
			if so.itemhistorymethod == "looter" then
				if so.itemhistoryway == "asc" then
					return a[3].player < b[3].player;
				else
					return a[3].player > b[3].player;
				end
			else
				if so.itemhistoryway == "asc" then
					return rt:GetTimestamp(a[2].key) < rt:GetTimestamp(b[2].key);
				else
					return rt:GetTimestamp(a[2].key) > rt:GetTimestamp(b[2].key);
				end
			end
		end	);

		RT_DetailFrameRaids.data = loot;
		RT_DetailFrameRaids.name = name;
		RT_DetailFrameRaids.maxlines = #loot;
		KA_RaidTracker_DetailScrollFramePlayer_Update();
		RT_DetailFrameRaidsText:SetText(name .. " (" .. #loot .. "/" .. totalItems .. "):");
		RT_DetailFrameRaids:Show();

	elseif ( frame.type == "events" ) then
		RT_DetailFramePlayers:Hide();
		RT_DetailFrameRaids:Hide();
		RT_DetailFrameEvents:Show();
		RT_DetailFrameItems:Hide();
		RT_DetailFrameEventsTabBoss:Show();
		RT_DetailFrameEventsTab1:SetWidth(163);
		RT_DetailFrameEventsTab1Middle:SetWidth(155);

		local events = {};
		if ( raid and raid.BossKills ) then
			for key, val in pairs(raid.BossKills) do
				tinsert(events, val);
			end
		end
		
		RT_DetailFrameEvents.raidid = raidid;
		RT_DetailFrameEvents.events = events;
		KA_RaidTracker_DetailScrollFrameEvents_Update();
		RT_DetailFrameEvents:Show();

	elseif ( frame.type == "playeritems" ) then
		RT_DetailFramePlayers:Hide();
		RT_DetailFrameRaids:Hide();
		RT_DetailFrameItems:Show();
		RT_DetailFrameEvents:Hide();
		local name = frame.player;

		local loot = { };
		for k, v in pairs(db.RaidLog) do
			for key, val in pairs(v.Loot) do
				if ( val.player == name ) then
					tinsert(
						loot,
						{
							["note"] = val.note,
							["player"] = val.player,
							["time"] = val.time,
							item = val.item,
							["ids"] = { k, key }
						}
					);
				end
			end
		end

		local numItems, numHidden = 0, 0;
		local keystoremove = {};
		table.sort( loot, KA_RaidTracker_CompareItems );
		for key, val in pairs(loot) do
			if ( KA_RaidTracker_RarityTable[val.item.c] >= db.SortOptions["playeritemsfilter"] ) then
				numItems = numItems + 1;
			else
				tinsert(keystoremove, key);
				numHidden = numHidden + 1;
			end
		end
		for key, val in pairs(keystoremove) do
			table.remove(loot, val);
		end
		RT_DetailFrameItems.raidid = raidid;
		RT_DetailFrameItems.loot = loot;
		KA_RaidTracker_DetailScrollFrameItems_Update();
		if ( numHidden == 0 ) then
			RT_DetailFrameItemsText:SetText(name .. "'s Loot (" .. numItems .. "):");
		else
			RT_DetailFrameItemsText:SetText(name .. "'s Loot (" .. numItems .. "/" .. numHidden + numItems .. "):");
		end

		RT_RarityDropdown:OnUpdate( );
	end
end

function KA_RaidTracker_DetailScrollFramePlayers_Update()
	local raidid = RT_DetailFramePlayers.raidid;
	local players = RT_DetailFramePlayers.players;
	local maxlines = getn(players);
	local line;
	local lineplusoffset;
	FauxScrollFrame_Update(RT_DetailFramePlayers, maxlines, 11, 18);
	for line=1, 11 do
		lineplusoffset = line + FauxScrollFrame_GetOffset(RT_DetailFramePlayers);
		if (lineplusoffset <= maxlines) then
			val = players[lineplusoffset];
			getglobal("RT_DetailFramePlayersLine" .. line).raidid = raidid;
			getglobal("RT_DetailFramePlayersLine" .. line).raidtitle = KA_RaidTracker_GetRaidTitle(raidid, 1);
			getglobal("RT_DetailFramePlayersLine" .. line).playername = val.name;
			local name = getglobal("RT_DetailFramePlayersLine" .. line .. "Name");
			local number = getglobal("RT_DetailFramePlayersLine" .. line .. "Number");
			local join = getglobal("RT_DetailFramePlayersLine" .. line .. "Join");
			local leave = getglobal("RT_DetailFramePlayersLine" .. line .. "Leave");
			if ( name ) then
				name:SetText(val.name);
				local iNumber = lineplusoffset;
				if ( iNumber < 10 ) then
					iNumber = "  " .. iNumber;
				end
				number:SetText(iNumber);
				if(KARaidTrackerDB.Options["TimeFormat"] == 1) then
					join:SetText(date("%H:%M", val["join"]));
				else
					join:SetText(date("%I:%M%p", val["join"]));
				end
				if ( val["leave"] == 99999999999 ) then
					leave:SetText("");
				else
					if(KARaidTrackerDB.Options["TimeFormat"] == 1) then
						leave:SetText(date("%H:%M", val["leave"]));
					else
						leave:SetText(date("%I:%M%p", val["leave"]));
					end
				end
				if ( KARaidTrackerDB.RaidLog[raidid].PlayerInfos[val.name] and KARaidTrackerDB.RaidLog[raidid].PlayerInfos[val.name].note ) then
					getglobal("RT_DetailFramePlayersLine" .. line .. "NoteButtonNormalTexture"):SetVertexColor(1, 1, 1);
				else
					getglobal("RT_DetailFramePlayersLine" .. line .. "NoteButtonNormalTexture"):SetVertexColor(0.5, 0.5, 0.5);
				end
				getglobal("RT_DetailFramePlayersLine" .. line .. "NoteButton"):Show();
				getglobal("RT_DetailFramePlayersLine" .. line .. "DeleteButton"):Show();
				getglobal("RT_DetailFramePlayersLine" .. line):Show();
			end
		else
			getglobal("RT_DetailFramePlayersLine" .. line):Hide();
		end
	end
	RT_DetailFramePlayers:Show();
end

function KA_RaidTracker_DetailScrollFrameItems_Update()
	local frame = RaidTrackerFrame;
	local db = KARaidTrackerDB;

	local raidid = RT_DetailFrameItems.raidid;
	local loot = RT_DetailFrameItems.loot;
	local raid = db.RaidLog[raidid]

	local numEntries = getn(loot);
	FauxScrollFrame_Update(RT_DetailFrameItems, numEntries, 5, 41);
	for i=1, 5 do
		local line = getglobal("RT_DetailFrameItemsLine" .. i);
		local itemid = i + FauxScrollFrame_GetOffset(RT_DetailFrameItems);

		if itemid > numEntries then
			line:Hide();
		else
			local item = loot[itemid]

			if frame.type == "items" then
				line.raidid = raidid;
				line.itemid = item.id;
				line.itemname = item.item.name;

				if item.item.count and item.item.count > 1 then
					getglobal("RT_DetailFrameItemsLine" .. i .. "Count"):Show();
					getglobal("RT_DetailFrameItemsLine" .. i .. "Count"):SetText(item.item.count);
				else
					getglobal("RT_DetailFrameItemsLine" .. i .. "Count"):Hide();
				end
				if item.item.icon then
					getglobal("RT_DetailFrameItemsLine" .. i .. "IconTexture"):SetTexture("Interface\\Icons\\" .. item.item.icon);
				else
					getglobal("RT_DetailFrameItemsLine" .. i .. "IconTexture"):SetTexture("Interface\\Icons\\INV_Misc_Gear_08");
				end
				local color = item.item.c;
				if color == "ff1eff00" then
					color = "ff005D00";
				end
				getglobal("RT_DetailFrameItemsLine" .. i .. "Description"):SetText("|c" .. color .. item.item.name);
				getglobal("RT_DetailFrameItemsLine" .. i .. "Looted"):SetText("Looted by: " .. KARaidTracker:ToStr(item.player));

				if item.note then
					getglobal("RT_DetailFrameItemsLine" .. i .. "NoteNormalTexture"):SetVertexColor(1, 1, 1);
				else
					getglobal("RT_DetailFrameItemsLine" .. i .. "NoteNormalTexture"):SetVertexColor(0.5, 0.5, 0.5);
				end
			elseif frame.type == "playeritems" then
				line.raidid = item.ids[1];
				line.itemid = item.ids[2];
				line.itemname = item.item.name;

				if ( item.item.count and item.item.count > 1 ) then
					getglobal("RT_DetailFrameItemsLine" .. i .. "Count"):Show();
					getglobal("RT_DetailFrameItemsLine" .. i .. "Count"):SetText(item.item.count);
				else
					getglobal("RT_DetailFrameItemsLine" .. i .. "Count"):Hide();
				end
				if ( item.item.icon ) then
					getglobal("RT_DetailFrameItemsLine" .. i .. "IconTexture"):SetTexture("Interface\\Icons\\" .. item.item.icon);
				else
					getglobal("RT_DetailFrameItemsLine" .. i .. "IconTexture"):SetTexture("Interface\\Icons\\INV_Misc_Gear_08");
				end
				local color = item.item.c;
				if ( color == "ff1eff00" ) then
					color = "ff005D00";
				end
				getglobal("RT_DetailFrameItemsLine" .. i .. "Description"):SetText("|c" .. color .. item.item.name);
				getglobal("RT_DetailFrameItemsLine" .. i .. "Looted"):SetText("Looted " .. KA_RaidTracker_GetRaidTitle(item.ids[1], 1));

				if ( item.note ) then
					getglobal("RT_DetailFrameItemsLine" .. i .. "NoteNormalTexture"):SetVertexColor(1, 1, 1);
				else
					getglobal("RT_DetailFrameItemsLine" .. i .. "NoteNormalTexture"):SetVertexColor(0.5, 0.5, 0.5);
				end
			end
			line:Show();
		end
	end
	RT_DetailFrameItems:Show();
end

function KA_RaidTracker_DetailScrollFramePlayer_Update()
	local data = RT_DetailFrameRaids.data;
	local name = RT_DetailFrameRaids.name;
	local maxlines = RT_DetailFrameRaids.maxlines;
	local line;
	local lineplusoffset;
	FauxScrollFrame_Update(RT_DetailFrameRaids, maxlines, 11, 18);
	for line=1, 11 do
		lineplusoffset = line+FauxScrollFrame_GetOffset(RT_DetailFrameRaids);
		if (lineplusoffset <= maxlines) then
			val = data[lineplusoffset];
			if ( RaidTrackerFrame.type == "player" ) then
				getglobal("RT_DetailFrameRaidsLine" .. line).raidid = val[1];
				getglobal("RT_DetailFrameRaidsLine" .. line).playername = name;
				getglobal("RT_DetailFrameRaidsLine" .. line).raidtitle = KA_RaidTracker_GetRaidTitle(val[1], 1);

				local iNumber = getn(KARaidTrackerDB.RaidLog)-val[1]+1;
				if ( iNumber < 10 ) then
					iNumber = "  " .. iNumber;
				end

				getglobal("RT_DetailFrameRaidsLine" .. line .. "MouseOverLeft"):Hide();
				getglobal("RT_DetailFrameRaidsLine" .. line .. "MouseOverRight"):Hide();

				getglobal("RT_DetailFrameRaidsLine" .. line .. "HitAreaLeft"):Hide();
				getglobal("RT_DetailFrameRaidsLine" .. line .. "HitAreaRight"):Hide();
				getglobal("RT_DetailFrameRaidsLine" .. line .. "HitArea"):Show();
				getglobal("RT_DetailFrameRaidsLine" .. line .. "Note"):Hide();
				getglobal("RT_DetailFrameRaidsLine" .. line .. "NoteButton"):Show();
				getglobal("RT_DetailFrameRaidsLine" .. line .. "DeleteButton"):Show();
				getglobal("RT_DetailFrameRaidsLine" .. line .. "DeleteText"):Show();

				getglobal("RT_DetailFrameRaidsLine" .. line .. "Number"):SetText(iNumber);
				getglobal("RT_DetailFrameRaidsLine" .. line .. "Name"):SetWidth(200);
				getglobal("RT_DetailFrameRaidsLine" .. line .. "Name"):SetText(KA_RaidTracker_GetRaidTitle(val[1], 1));

				if ( val[2].PlayerInfos[name] and val[2].PlayerInfos[name].note ) then
					getglobal("RT_DetailFrameRaidsLine" .. line .. "NoteButtonNormalTexture"):SetVertexColor(1, 1, 1);
				else
					getglobal("RT_DetailFrameRaidsLine" .. line .. "NoteButtonNormalTexture"):SetVertexColor(0.5, 0.5, 0.5);
				end
			elseif ( RaidTrackerFrame.type == "itemhistory" ) then
				getglobal("RT_DetailFrameRaidsLine" .. line).raidid = val[1];

				local iNumber = getn(KARaidTrackerDB.RaidLog)-val[1]+1;
				if ( iNumber < 10 ) then
					iNumber = "  " .. iNumber;
				end
				getglobal("RT_DetailFrameRaidsLine" .. line .. "MouseOver"):Hide();

				getglobal("RT_DetailFrameRaidsLine" .. line .. "HitAreaLeft"):Show();
				getglobal("RT_DetailFrameRaidsLine" .. line .. "HitAreaRight"):Show();
				getglobal("RT_DetailFrameRaidsLine" .. line .. "HitArea"):Hide();

				getglobal("RT_DetailFrameRaidsLine" .. line .. "NoteButton"):Hide();
				getglobal("RT_DetailFrameRaidsLine" .. line .. "Note"):Show();
				getglobal("RT_DetailFrameRaidsLine" .. line .. "DeleteButton"):Hide();
				getglobal("RT_DetailFrameRaidsLine" .. line .. "DeleteText"):Hide();

				getglobal("RT_DetailFrameRaidsLine" .. line .. "Number"):SetText(iNumber);
				getglobal("RT_DetailFrameRaidsLine" .. line .. "Name"):SetWidth(130);
				getglobal("RT_DetailFrameRaidsLine" .. line .. "Name"):SetText(KA_RaidTracker_GetRaidTitle(val[1], 1));
				getglobal("RT_DetailFrameRaidsLine" .. line .. "Note"):SetText(val[3].player);
			end
			getglobal("RT_DetailFrameRaidsLine" .. line):Show();
		else
			getglobal("RT_DetailFrameRaidsLine" .. line):Hide();
		end
	end
	RT_DetailFrameRaids:Show();
end

function KA_RaidTracker_DetailScrollFrameEvents_Update()
	local events = RT_DetailFrameEvents.events;
	local maxlines = getn(events);
	local line;
	local lineplusoffset;
	FauxScrollFrame_Update(RT_DetailFrameEvents, maxlines, 11, 18);
	for line=1, 11 do
		lineplusoffset = line+FauxScrollFrame_GetOffset(RT_DetailFrameEvents);
		if (lineplusoffset <= maxlines) then
			val = events[lineplusoffset];
			getglobal("RT_DetailFrameEventsLine" .. line .. "MouseOver"):Hide();
			getglobal("RT_DetailFrameEventsLine" .. line .. "HitArea"):Show();
			getglobal("RT_DetailFrameEventsLine" .. line .. "Boss"):SetText(val.boss);
			getglobal("RT_DetailFrameEventsLine" .. line .. "Time"):SetText(val.time);
			getglobal("RT_DetailFrameEventsLine" .. line):Show();
		else
			getglobal("RT_DetailFrameEventsLine" .. line):Hide();
		end
	end
	RT_DetailFrameEvents:Show();
end


function KA_RaidTrackerItem_SetHyperlink( frame )
	local db = KARaidTrackerDB
	if not (frame and frame.raidid and frame.itemid) then return end
	local item = db.RaidLog[frame.raidid].Loot[frame.itemid]

	GameTooltip:SetOwner(frame, "ANCHOR_RIGHT");
	if GetItemInfo("item:" .. item.item.id) then
		GameTooltip:SetHyperlink("item:" .. item.item.id);
	else
		rl, gl, bl = KARaidTracker:ColorToRGB(strsub(item.item.c, 3));
		GameTooltip:AddLine(item.item.name, rl, gl, bl);
		GameTooltip:AddLine("This item is not in your cache, you can try to get the information by rightclicking the item (This may result in a disconnect!)", 1, 1, 1);
	end

	GameTooltip:AddLine("Time: "..item.time, 1, 1, 0);
	if item.zone then	GameTooltip:AddLine("Zone: "..item.zone, 1, 1, 0)	end
	if item.boss then	GameTooltip:AddLine("Boss: "..item.boss, 1, 1, 0)	end
	if item.costs then	GameTooltip:AddLine("Costs: "..item.costs, 1, 1, 0) end
	if db.Options.DebugMode == 1 then
		if item.item.class then		GameTooltip:AddLine("Class: "..item.item.class, 1, 1, 0) end
		if item.item.subclass then	GameTooltip:AddLine("SubClass: "..item.item.subclass, 1, 1, 0) end
	end

	GameTooltip:Show();
end

function KA_RaidTrackerItem_GetChatHyperlink( frame )
	local db = KARaidTrackerDB
	if not (frame and frame.raidid and frame.itemid) then return end
	local item = db.RaidLog[frame.raidid].Loot[frame.itemid];

	if item and IsShiftKeyDown() and type(WIM_API_InsertText)=="function" then
		WIM_API_InsertText("|c"..item.item.c.."|Hitem:"..item.item.id.."|h["..item.item.name.."]|h|r");
	end
	if item and IsShiftKeyDown() and ChatFrameEditBox:IsVisible() then
		ChatFrameEditBox:Insert("|c"..item.item.c.."|Hitem:"..item.item.id.."|h["..item.item.name.."]|h|r");
	end
end


function KA_RaidTracker_AddWipe()
	local db = KARaidTrackerDB
	KarmaUtility:Debug("WIPED");
	KA_RaidTracker_LastWipe = GetTime()+db.Options.WipeCooldown; -- wait for 120 seconds
	if db.RaidLog[db.Options.CurrentRaid].wipes == nil then
		db.RaidLog[db.Options.CurrentRaid].wipes = {};
	end;
	tinsert(db.RaidLog[db.Options.CurrentRaid].wipes,time());
	KA_RaidTracker_Print("Wipe has been recorded!", 1, 1, 0);
end;

function KA_RaidTracker_DoRaidIdCheck()
	local db = KARaidTrackerDB
	if not db.Options.CurrentRaid then return end

	local savedInstances = GetNumSavedInstances();
	local instanceName, instanceID, instanceReset;
	if ( savedInstances > 0 ) then
		for i=1, MAX_RAID_INFOS do
			if ( i <=  savedInstances) then
				instanceName, instanceID, instanceReset = GetSavedInstanceInfo(i);
				if (db.RaidLog[db.Options.CurrentRaid].zone == instanceName) then
					db.RaidLog[db.Options.CurrentRaid].instanceid = instanceID;
					--db.RaidLog[db.Options.CurrentRaid]["instancereset"] = instanceReset;
				end;
			end

		end
	end
end;


function KA_RaidTracker_SelectItem(name)
	KA_RaidTracker_GetPage();
	RaidTrackerFrame.type = "itemhistory";
	RaidTrackerFrame.itemname = name;
	RaidTrackerFrame.selected = nil;
	KA_RaidTracker_Update();
	KA_RaidTracker_UpdateView();
end

function KA_RaidTracker_GetPage()
	if RaidTrackerFrame.type or RaidTrackerFrame.itemname or RaidTrackerFrame.selected or RaidTrackerFrame.player then
		tinsert(RT_Buttons.LastPage, {
				type		= RaidTrackerFrame.type,
				itemname	= RaidTrackerFrame.itemname,
				selected	= RaidTrackerFrame.selected,
				player		= RaidTrackerFrame.player
			}
		);
	end
	if #RT_Buttons.LastPage > 0 then
		RaidTrackerFrameBackButton:Enable();
	else
		RaidTrackerFrameBackButton:Disable();
	end
end

function KA_RaidTracker_GoBack()
	local t = table.remove(RT_Buttons.LastPage);
	if t then
		RaidTrackerFrame.type = t.type;
		RaidTrackerFrame.itemname = t.itemname;
		RaidTrackerFrame.selected = t.selected;
		RaidTrackerFrame.player = t.player;
		KA_RaidTracker_Update();
		KA_RaidTracker_UpdateView();
	end
	if #RT_Buttons.LastPage > 0 then
		RaidTrackerFrameBackButton:Enable();
	else
		RaidTrackerFrameBackButton:Disable();
	end
end


function KA_RaidTrackerAddGuild()
	local db = KARaidTrackerDB

	if(db.Options.CurrentRaid) then
		SetGuildRosterShowOffline(false);
	 for i = 1, GetNumGuildMembers() do
		local name, rank, rankIndex, level, class, zone, group, note, officernote, online = GetGuildRosterInfo(i);
	   -- KarmaUtility:Debug("GUILD", name, online);
		if( online ~= db.Online[name] ) then
		  if( online ) then
			tinsert(db.RaidLog[db.Options.CurrentRaid].Join,
			{
			   ["player"] = name,
			   ["time"] = KARaidTracker:GetDisplayTime()
			}
			);
			db.Online[name] = online;
			KarmaUtility:Debug("GUILD-ONLINE", name);
		  elseif ( not online and db.Online[name]) then
						tinsert(db.RaidLog[db.Options.CurrentRaid].Leave,
							{
								["player"] = name,
								["time"] = KARaidTracker:GetDisplayTime()
							}
						);
						db.Online[name] = online;
						KarmaUtility:Debug("GUILD-OFFLINE", name);
			end
		end
	 end
  end
	KA_RaidTracker_Update();
	KA_RaidTracker_UpdateView();
end



-- Time offset handling

function KA_RaidTracker_GameTimeFrame_Update()
	-- call orginal blizz function
	KA_RaidTracker_GameTimeFrame_Update_Original();
	local db = KARaidTrackerDB;

	local hour, minute = GetGameTime();
	local time = ((hour * 60) + minute) * 60;

	if( not db.Options["TimeOffsetStatus"] ) then
		db.Options["TimeOffsetStatus"] = time;
	elseif ( db.Options["TimeOffsetStatus"] ~= time ) then
		local ltimea = date("*t");
		local ltime = (((ltimea["hour"] * 60) + ltimea["min"]) * 60 + ltimea["sec"]) + (db.Options["Timezone"] * 3600);

		if( time > ltime ) then
			local timediff = time - ltime;
			if(timediff >= 43200) then
				KA_RaidTracker_TimeOffset = timediff - 86400;
			else
				KA_RaidTracker_TimeOffset = timediff;
			end
		elseif ( time < ltime ) then
			local timediff = ltime - time;
			if(timediff >= 43200) then
				KA_RaidTracker_TimeOffset = 86400 - timediff;
			else
				KA_RaidTracker_TimeOffset = timediff * -1;
			end
		else
			KA_RaidTracker_TimeOffset = 0;
		end

		KarmaUtility:Debug("KA_RaidTracker_TimeOffset", KA_RaidTracker_TimeOffset);
		GameTimeFrame_Update = KA_RaidTracker_GameTimeFrame_Update_Original;
		db.Options["TimeOffsetStatus"] = nil;
	end
end

function KA_RaidTracker_GetGameTimeOffset()
	if KARaidTrackerDB.Options["TimeOffsetStatus"] then
		return;
	end
	-- hook wow GameTimeFrame_Update function
	if (not KA_RaidTracker_GameTimeFrame_Update_Original) then
		KA_RaidTracker_GameTimeFrame_Update_Original = GameTimeFrame_Update;
	end
	GameTimeFrame_Update = KA_RaidTracker_GameTimeFrame_Update;
end

function RT_UpdateFrame_OnUpdate( frame, elapsed )
	if ( frame.time ) then
		frame.time = frame.time + elapsed;
		if ( frame.time > 2 ) then
			frame.time = nil;
			for k, v in pairs(KA_RaidTracker_Events) do
				KARaidTracker:Debug("Running", unpack(v));
				KARaidTracker:OnEvent(unpack(v));
			end
		end
	end
end

